Package: openCyto
Type: Package
Title: Hierarchical Gating Pipeline for flow cytometry data
Version: 2.24.0
Date: 2026-03-12
Author: Mike Jiang, John Ramey, Greg Finak, Raphael Gottardo
Maintainer: Mike Jiang <mike@ozette.com>
Description: This package is designed to facilitate the automated gating
    methods in sequential way to mimic the manual gating strategy.
License: AGPL-3.0-only
LazyLoad: yes
Imports: methods, Biobase, BiocGenerics, flowCore(>= 1.99.17), flowViz,
        ncdfFlow(>= 2.11.34), flowWorkspace(>= 3.99.1), flowClust(>=
        3.11.4), RBGL, graph, data.table, RColorBrewer, grDevices
Suggests: flowWorkspaceData, knitr, rmarkdown, markdown, testthat,
        utils, tools, parallel, ggcyto, CytoML, flowStats(>= 4.5.2),
        MASS
biocViews: ImmunoOncology, FlowCytometry, DataImport, Preprocessing,
        DataRepresentation
Encoding: UTF-8
VignetteBuilder: knitr
RoxygenNote: 7.3.2
LinkingTo: cpp11, BH(>= 1.62.0-1)
git_url: https://git.bioconductor.org/packages/openCyto
git_branch: RELEASE_3_23
git_last_commit: 8a4e8f8
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 23:41:52 UTC; biocbuild
Depends: R (>= 3.5.0)
