# Generated by roxygen2: do not edit by hand

S3method(as.data.table,gatingTemplate)
S3method(gt_gating,dummyMethod)
S3method(gt_gating,gatingTemplate)
S3method(pop_add,ocRectRefGate)
export(CytoExploreR_.argDeparser)
export(CytoExploreR_.preprocess_csv)
export(CytoExploreR_.preprocess_row)
export(add_pop)
export(add_pop_init)
export(delete.helperGates)
export(fast_rlm)
export(flowClust.1d)
export(flowClust.2d)
export(gate_flowClust_1d)
export(gate_flowClust_2d)
export(gate_flowclust_1d)
export(gate_flowclust_2d)
export(gate_mindensity)
export(gate_mindensity2)
export(gate_quad_sequential)
export(gate_quad_tmix)
export(gate_quantile)
export(gating)
export(gatingTemplate)
export(generate_gate_template)
export(get.helperGates)
export(gh_generate_template)
export(gs_add_gating_method)
export(gs_add_gating_method_init)
export(gs_remove_gating_method)
export(gt_delete_helpergates)
export(gt_gating)
export(gt_get_children)
export(gt_get_gate)
export(gt_get_helpergates)
export(gt_get_nodes)
export(gt_get_parent)
export(gt_list_methods)
export(gt_toggle_helpergates)
export(listgtMethods)
export(mindensity)
export(mindensity2)
export(prior_flowClust)
export(prior_flowclust)
export(quadGate.seq)
export(quadGate.tmix)
export(quantileGate)
export(registerPlugins)
export(register_plugins)
export(remove_pop)
export(templateGen)
export(toggle.helperGates)
exportClasses(gatingTemplate)
exportMethods(getChildren)
exportMethods(getGate)
exportMethods(getNodes)
exportMethods(getParent)
exportMethods(plot)
exportMethods(show)
import(data.table)
import(flowWorkspace)
import(utils)
importClassesFrom(Biobase,AssayData)
importClassesFrom(flowCore,ellipsoidGate)
importClassesFrom(flowCore,filterList)
importClassesFrom(flowCore,polygonGate)
importClassesFrom(flowCore,rectangleGate)
importClassesFrom(graph,graph)
importClassesFrom(graph,graphBase)
importClassesFrom(graph,graphNEL)
importClassesFrom(methods,ANY)
importClassesFrom(methods,character)
importClassesFrom(methods,data.frame)
importClassesFrom(methods,environment)
importClassesFrom(methods,list)
importClassesFrom(methods,logical)
importClassesFrom(methods,matrix)
importClassesFrom(methods,missing)
importClassesFrom(methods,numeric)
importClassesFrom(methods,oldClass)
importClassesFrom(ncdfFlow,ncdfFlowList)
importClassesFrom(ncdfFlow,ncdfFlowSet)
importFrom(BiocGenerics,dims)
importFrom(RBGL,tsort)
importFrom(RColorBrewer,brewer.pal)
importFrom(data.table,fwrite)
importFrom(flowClust,dmvt)
importFrom(flowClust,dmvtmix)
importFrom(flowClust,flowClust)
importFrom(flowClust,getEstimates)
importFrom(flowClust,rbox)
importFrom(flowClust,tmixFilter)
importFrom(flowCore,"%in%")
importFrom(flowCore,"exprs<-")
importFrom(flowCore,"filterDetails<-")
importFrom(flowCore,"parameters<-")
importFrom(flowCore,colnames)
importFrom(flowCore,exprs)
importFrom(flowCore,filters)
importFrom(flowCore,getChannelMarker)
importFrom(flowCore,identifier)
importFrom(flowCore,parameters)
importFrom(flowCore,polygonGate)
importFrom(flowCore,quadGate)
importFrom(flowCore,rectangleGate)
importFrom(flowWorkspace,cf_lock)
importFrom(flowWorkspace,cf_unlock)
importFrom(flowWorkspace,gh_get_pop_paths)
importFrom(flowWorkspace,gh_get_transformations)
importFrom(flowWorkspace,gh_pop_get_data)
importFrom(flowWorkspace,gh_pop_get_gate)
importFrom(flowWorkspace,gh_pop_is_negated)
importFrom(flowWorkspace,gh_pop_set_visibility)
importFrom(flowWorkspace,gs_get_pop_paths)
importFrom(flowWorkspace,gs_pop_get_children)
importFrom(flowWorkspace,identifier)
importFrom(flowWorkspace,pData)
importFrom(flowWorkspace,realize_view)
importFrom(flowWorkspace,sampleNames)
importFrom(flowWorkspace,transform)
importFrom(flowWorkspace,transformerList)
importFrom(grDevices,chull)
importFrom(graph,"edgeData<-")
importFrom(graph,"edgeDataDefaults<-")
importFrom(graph,"nodeData<-")
importFrom(graph,"nodeDataDefaults<-")
importFrom(graph,addEdge)
importFrom(graph,edgeData)
importFrom(graph,edgeDataDefaults)
importFrom(graph,edges)
importFrom(graph,graphNEL)
importFrom(graph,inEdges)
importFrom(graph,nodeData)
importFrom(graph,nodeDataDefaults)
importFrom(graph,nodes)
importFrom(graph,subGraph)
importFrom(methods,as)
importFrom(methods,coerce)
importFrom(methods,show)
useDynLib(openCyto,.registration = TRUE)
