# Generated by cpp11: do not edit by hand

Cdqrls <- function(x, y, tol) {
  .Call(`_openCyto_Cdqrls`, x, y, tol)
}

collapseData <- function(mat_list, colnames) {
  .Call(`_openCyto_collapseData`, mat_list, colnames)
}

rlm_cpp <- function(x, y, maxit) {
  .Call(`_openCyto_rlm_cpp`, x, y, maxit)
}

solve_LSAP_cpp <- function(mat) {
  .Call(`_openCyto_solve_LSAP_cpp`, mat)
}
