% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{fcFilterList-class}
\alias{fcFilterList-class}
\title{a class that extends \code{filterList} class.}
\description{
Each filter in the filterList must extends the \code{fcFilter} class
}
