% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{gtSubsets-class}
\alias{gtSubsets-class}
\title{A class representing a group of cell populations.}
\description{
It extends \code{gtPopulation} class.
}
