% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add.R
\docType{class}
\name{ocRectangleGate-class}
\alias{ocRectangleGate-class}
\title{the class that carries event indices as well}
\description{
the class that carries event indices as well
}
