% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fcTree-methods.R
\name{getGate,fcTree,character-method}
\alias{getGate,fcTree,character-method}
\title{get gates saved in \code{fcTree}}
\usage{
\S4method{getGate}{fcTree,character}(obj, y, ...)
}
\arguments{
\item{obj}{\code{fcTree}}

\item{y}{\code{character} node name}

\item{...}{other arguments (not used)}
}
\description{
get gates saved in \code{fcTree}
}
