% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{boolMethod-class}
\alias{boolMethod-class}
\title{A class to represent a boolean gating method.}
\description{
It extends \code{refGate} class.
}
